#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Thu Mar 20 16:58:45 2014

GeTCal
Great Tool for Calibration

This is now the final version that makes the 
Teff and [Fe/H] calibrations, plots the fitting steps and prints a
txt file compatible with TMCalc

v1.04 2014-11-07
See CHANGELOG

@author: gteixeira
"""


import numpy as np
from numpy import linalg
from scipy import stats
import math as math
import matplotlib.pyplot as plt
import robust as robust
import os
from GeTCal_config import *


def ares_output_reader(ares_out_sel):
  output_lines=[]  
  for line in open(ares_out_sel,'r'):
    line=line.split()
    output_lines.append([line[0],line[4]])
  output_lines=[each for each in output_lines if each !=[] ]
  output_lines=np.array(output_lines)
  return output_lines  
  
def get_element_read(line_list,sel_line):
  line_listed=[]
  for line in open(line_list,'r'):
    line_listed.append(line.split())
  line_listed=line_listed[2:]
  for line in line_listed:
      if str(line[0])==str(sel_line):
          ele=line[3]
  return ele
  
def allow_ratios(line_list_sel,selected_lines):
  output=[]
  output_lines=[]  
  line_listed=[] 
  for line in open(line_list_sel,'r'):
    line_listed.append(line.split())
  for element in line_listed[2:]:
    output_lines.append([element[0],element[1]])
  match=np.array(output_lines)
  for i in range(0,len(match)-1):
    for j in range(i+1,len(match)):
      if abs(float(match[j,0])-float(match[i,0]))<75 and abs(float(match[j,1])-float(match[i,1]))>3:
        output.append((float(match[i,0]),float(match[j,0]))) 
  return output
  

def allow_fe(line_list):
  output=[]  
  output_lines=[]
  line_listed=[] 
  for line in open(line_list,'r'):
    line_listed.append(line.split())  
  for element in line_listed[2:]:
      if str(element[3])=='FeI':
          output_lines.append([element[0],element[3]])
  for j in output_lines:
    if j not in output:
      output.append(j)
  return output
  
  
def fe_aux(ares_outputed,line_listed):  
  line_listed=np.array(line_listed)
  ares_outputed=np.reshape(ares_outputed,len(ares_outputed)*2)
  ares_outputed=np.reshape(ares_outputed,(len(ares_outputed)/2,-1))
  match=[]
  i=0
  while i < len(line_listed):
    count=0     
    j=0
    while j < len(ares_outputed):
      if str(ares_outputed[j][0]) == str(line_listed[i][0]) and 20<float(ares_outputed[j][1])<75:
            match+=[line_listed[i][0],float(ares_outputed[j][1])] 
            count=+1    
      j+=1    
    if count==0:
      match+=[line_listed[i][0],-1]
    i+=1
  match=np.reshape(match,(len(match)/2,-1))    
  return match  
  
  
def ratiocalc(ares_outputed,line_listed):  
  line_listed=np.array(line_listed)
  ares_outputed=np.reshape(ares_outputed,len(ares_outputed)*2)
  ares_outputed=np.reshape(ares_outputed,(len(ares_outputed)/2,-1))
  match=[]
  i=0
  while i < len(line_listed):
    count=0     
    j=0
    while j < len(ares_outputed):
      if str(ares_outputed[j][0]) == str(line_listed[i][0]):
        k=j+1
        while k<len(ares_outputed):
          if str(ares_outputed[k][0]) == str(line_listed[i][1]) and float(ares_outputed[k][1])>5 and float(ares_outputed[j][1])>5:
            count+=1
            match+=[line_listed[i][0],line_listed[i][1],float(ares_outputed[j][1])/float(ares_outputed[k][1])] 
          k+=1      
      j+=1    
    if count==0:
      match+=[line_listed[i][0],line_listed[i][1],-1]
    i+=1
  match=np.reshape(match,(len(match)/3,-1))  
  return match
  
def param_reader(param_file):
  output_list=[]  
  for line in open(param_file,'r'):
    line=line.split()
    output_list.append([float(line[1]),float(line[2]),float(line[3]),float(line[4]),float(line[5]),float(line[6]),float(line[7]),float(line[8])]) 
  return output_list
  

def ultimate_plotter2(data,option,line_list):
  plt.figure(figsize=(10,15)) 
  ratios_1=np.array([np.array((float(data[i][2]),1)) for i in range(0,len(data)) if float(data[i][2]) != -1 and float(data[i][2]) < 100 and float(data[i][2]) > 0.01 and float(data[i][2])<2*np.mean(data[:][2])])
  temperatures=np.array([np.array((float(data[i][3]),float(data[i][4]))) for i in range(0,len(data)) if float(data[i][2]) != -1 and float(data[i][2]) < 100 and float(data[i][2]) > 0.01 and float(data[i][2])<2*np.mean(data[:][2])])
  output=[]
  if np.any(ratios_1)==True:
      quart1=stats.scoreatpercentile(ratios_1[:,0],25)
      quart3=stats.scoreatpercentile(ratios_1[:,0],75)
      iqr=quart3-quart1
      lowqr=quart1-1.5*iqr
      highqr=quart3+1.5*iqr
      ratios_temp=ratios_1      
      ratios_1=np.array([ratios_1[i,:] for i in range(0,len(ratios_1)) if ratios_1[i,0] <highqr and ratios_1[i,0] > lowqr])      
      temperatures=np.array([temperatures[i,:] for i in range(0,len(temperatures)) if ratios_temp[i,0] <highqr and ratios_temp[i,0] > lowqr])
      poli_r=robust.polyfit(ratios_1[:,0],temperatures[:,0],3)
      ypred_r=(np.polyval(poli_r,ratios_1[:,0])-temperatures[:,0])
      r_res=np.std(np.polyval(poli_r,ratios_1[:,0])-temperatures[:,0])
      poli2_r=np.poly1d(poli_r)
      poliy_r=np.linspace(min(ratios_1[:,0]),max(ratios_1[:,0]),100)
      ratios_1=np.array([ratios_1[i,:] for i in range(0,len(ratios_1)) if abs((ypred_r[i])) <=2*r_res])      
      temperatures=np.array([temperatures[i,:] for i in range(0,len(temperatures)) if abs((ypred_r[i])) <=2*r_res])
      if np.any(ratios_1)==True:    
          w_r = np.polyfit(ratios_1[:,0],temperatures[:,0],1)
          r_lin_res=np.std(np.polyval(w_r,ratios_1[:,0])-temperatures[:,0])    
          linear_r=[]
          lin2_r=[]     
          for number in np.linspace(min(ratios_1[:,0]),max(ratios_1[:,0]),100):
            linear_r.append(float(w_r[0])*number+float(w_r[1]))
            lin2_r.append(number)
          poli_r=robust.polyfit(ratios_1[:,0],temperatures[:,0],3)  
          ypred_r=(np.polyval(poli_r,ratios_1[:,0])-temperatures[:,0])    
          r_res=np.std(np.polyval(poli_r,ratios_1[:,0])-temperatures[:,0])    
          poli2_r=np.poly1d(poli_r)
          poliy_r=np.linspace(min(ratios_1[:,0]),max(ratios_1[:,0]),100)
      if np.any(ratios_1)==True:
        output.append((option,int(r_lin_res),len(temperatures[:,0]),data[0][0],data[0][1],get_element_read(line_list,float(data[0][0])),get_element_read(line_list,float(data[0][1])),format(w_r[1], '.2f'),format(w_r[0], '.2f'),format(0, '.2f'),format(0, '.2f'),int(float(min(temperatures[:,0]))),int(float(max(temperatures[:,0]))),format(min(ratios_1[:,0]), '.2f'),format(max(ratios_1[:,0]), '.2f'),format(np.mean(ratios_1[:,0]), '.2f'),1))
        output.append((option,int(r_res),len(temperatures[:,0]),data[0][0],data[0][1],get_element_read(line_list,float(data[0][0])),get_element_read(line_list,float(data[0][1])),format(poli_r[3], '.2f'),format(poli_r[2], '.2f'),format(poli_r[1], '.2f'),format(poli_r[0], '.2f'),int(float(min(temperatures[:,0]))),int(float(max(temperatures[:,0]))),format(min(ratios_1[:,0]), '.2f'),format(max(ratios_1[:,0]), '.2f'),format(np.mean(ratios_1[:,0]), '.2f'),2))
        plt.subplot(3,1,1)
        plt.errorbar(ratios_1[:,0],temperatures[:,0],yerr=temperatures[:,1],fmt='bo')      
        plt.plot(ratios_1[:,0],temperatures[:,0],'bo',lin2_r[:],linear_r[:],'r-',poliy_r,poli2_r(poliy_r),'g-')
        plt.xlabel('R',fontsize=20)      
        plt.legend(['Stars: '+str(len(temperatures[:,0])),'P_sigma: '+str("%.0f" % r_res), 'L_sigma: '+str("%.0f" % r_lin_res)],prop={'size':12})
        plt.ylabel('Teff (K)',fontsize=20)    
        plt.ylim((4300,7200))
        plt.xlim((min(ratios_1[:,0])-min(ratios_1[:,0])*0.2,max(ratios_1[:,0])+0.2*max(ratios_1[:,0])))
      ratios_1=np.array([np.array((float(1)/(float(data[i][2])),1)) for i in range(0,len(data)) if float(data[i][2]) != -1 and float(data[i][2]) < 100 and float(data[i][2]) > 0.01])
      ratios_1_r=np.array([((float(data[i][2])),1) for i in range(0,len(data)) if float(data[i][2]) != -1 and float(data[i][2]) < 100 and float(data[i][2]) > 0.01])
      temperatures=np.array([np.array((float(data[i][3]),float(data[i][4]))) for i in range(0,len(data)) if float(data[i][2]) != -1 and float(data[i][2]) < 100 and float(data[i][2]) > 0.01])
      quart1=stats.scoreatpercentile(ratios_1[:,0],25)
      quart3=stats.scoreatpercentile(ratios_1[:,0],75)
      iqr=quart3-quart1
      lowqr=quart1-1.5*iqr
      highqr=quart3+1.5*iqr
      ratios_temp=ratios_1
      ratios_1=np.array([ratios_1[i,:] for i in range(0,len(ratios_1)) if ratios_1[i,0] <highqr and ratios_1[i,0] > lowqr])      
      temperatures=np.array([temperatures[i,:] for i in range(0,len(temperatures)) if ratios_temp[i,0] <highqr and ratios_temp[i,0] > lowqr])
      ratios_1_r=np.array([ratios_1_r[i,:] for i in range(0,len(ratios_1)) if ratios_1[i,0] <highqr and ratios_1[i,0] > lowqr])
      poli_1r=robust.polyfit(ratios_1[:,0],temperatures[:,0],3)    
      ypred_1r=(np.polyval(poli_1r,ratios_1[:,0])-temperatures[:,0])    
      inv_r_res=np.std(np.polyval(poli_1r,ratios_1[:,0])-temperatures[:,0])   
      poli2_1r=np.poly1d(poli_1r)
      poliy_1r=np.linspace(min(ratios_1[:,0]),max(ratios_1[:,0]),100)
      ratios_1=np.array([ratios_1[i,:] for i in range(0,len(ratios_1)) if abs((ypred_1r[i])) <=2*inv_r_res])      
      ratios_1_r=np.array([ratios_1_r[i,:] for i in range(0,len(ratios_1_r)) if abs((ypred_1r[i])) <=2*inv_r_res])       
      temperatures=np.array([temperatures[i,:] for i in range(0,len(temperatures)) if abs((ypred_1r[i])) <=2*inv_r_res]) 
      if np.any(temperatures)==True:
          w_r = robust.polyfit(ratios_1[:,0],temperatures[:,0],1)
          if (type(w_r) == int) != True and np.any(ratios_1_r) == True:
              line_error_1r=np.std(np.polyval(w_r,ratios_1[:,0])-temperatures[:,0])   
              linear_r=[]
              lin2_r=[]      
              for number in np.linspace(min(ratios_1[:,0]),max(ratios_1[:,0]),100):
                  linear_r.append(float(w_r[0])*number+float(w_r[1]))
                  lin2_r.append(number)
              poli_1r=robust.polyfit(ratios_1[:,0],temperatures[:,0],3)     
              ypred_1r=(np.polyval(poli_1r,ratios_1[:,0])-temperatures[:,0])    
              inv_r_res=np.std(np.polyval(poli_1r,ratios_1[:,0])-temperatures[:,0])         
              
              if np.any(ratios_1)==True:
                output.append((option,int(line_error_1r),len(temperatures[:,0]),data[0][0],data[0][1],get_element_read(line_list,float(data[0][0])),get_element_read(line_list,float(data[0][1])),format(w_r[1], '.2f'),format(w_r[0], '.2f'),format(0, '.2f'),format(0, '.2f'),int(float(min(temperatures[:,0]))),int(float(max(temperatures[:,0]))),format(min(1/ratios_1_r[:,0]), '.2f'),format(max(1/ratios_1_r[:,0]), '.2f'),format(np.mean(1/ratios_1_r[:,0]), '.2f'),3))
                output.append((option,int(inv_r_res),len(temperatures[:,0]),data[0][0],data[0][1],get_element_read(line_list,float(data[0][0])),get_element_read(line_list,float(data[0][1])),format(poli_1r[3], '.2f'),format(poli_1r[2], '.2f'),format(poli_1r[1], '.2f'),format(poli_1r[0], '.2f'),int(float(min(temperatures[:,0]))),int(float(max(temperatures[:,0]))),format(min(1/ratios_1_r[:,0]), '.2f'),format(max(1/ratios_1_r[:,0]), '.2f'),format(np.mean(1/ratios_1_r[:,0]), '.2f'),4))      
                plt.subplot(3,1,2)
                plt.errorbar(ratios_1[:,0],temperatures[:,0],yerr=temperatures[:,1],fmt='bo')
                plt.plot(ratios_1[:,0],temperatures[:,0],'bo',lin2_r[:],linear_r[:],'r-',poliy_1r,poli2_1r(poliy_1r),'g-')      
                plt.xlabel('1/R',fontsize=20)     
                plt.ylabel('Teff (K)',fontsize=20)    
                plt.legend(['Stars: '+str(len(temperatures[:,0])),'P_sigma: '+str("%.0f" % inv_r_res), 'L_sigma: '+str("%.0f" % line_error_1r)],prop={'size':12})
                plt.ylim((4300,7200))
                plt.xlim((min(ratios_1[:,0])-min(ratios_1[:,0])*0.2,max(ratios_1[:,0])+max(ratios_1[:,0])*0.2))      
      ratios_1=np.array([np.array((np.log10(float(data[i][2])),1)) for i in range(0,len(data)) if float(data[i][2]) != -1 and float(data[i][2]) < 100 and float(data[i][2]) > 0.01])
      temperatures=np.array([np.array((float(data[i][3]),float(data[i][4]))) for i in range(0,len(data)) if float(data[i][2]) != -1 and float(data[i][2]) < 100 and float(data[i][2]) > 0.01])
      quart1=stats.scoreatpercentile(ratios_1[:,0],25)
      quart3=stats.scoreatpercentile(ratios_1[:,0],75)
      iqr=quart3-quart1
      lowqr=quart1-1.5*iqr
      highqr=quart3+1.5*iqr
      ratios_temp=ratios_1
      ratios_1=np.array([ratios_1[i,:] for i in range(0,len(ratios_1)) if ratios_1[i,0] <highqr and ratios_1[i,0] > lowqr])      
      temperatures=np.array([temperatures[i,:] for i in range(0,len(temperatures)) if ratios_temp[i,0] <highqr and ratios_temp[i,0] > lowqr]) 
      ratios_1_r=np.array([ratios_1[i,:] for i in range(0,len(ratios_1)) if ratios_1[i,0] <highqr and ratios_1[i,0] > lowqr])
      poli_logr=robust.polyfit(ratios_1[:,0],temperatures[:,0],3)
      ypred_log=(np.polyval(poli_logr,ratios_1[:,0])-temperatures[:,0])
      logr_res=np.std(np.polyval(poli_logr,ratios_1[:,0])-temperatures[:,0])    
      poli2_logr=np.poly1d(poli_logr)
      poliy_logr=np.linspace(min(ratios_1[:,0]),max(ratios_1[:,0]),100)
      ratios_1=np.array([ratios_1[i,:] for i in range(0,len(ratios_1)) if abs((ypred_log[i])) <=2*logr_res])      
      ratios_1_r=np.array([ratios_1_r[i,:] for i in range(0,len(ratios_1)) if abs((ypred_log[i])) <=2*logr_res])    
      temperatures=np.array([temperatures[i,:] for i in range(0,len(temperatures)) if abs((ypred_log[i])) <=2*logr_res])      
      if np.any(ratios_1)==True:
          w_r = robust.polyfit(ratios_1[:,0],temperatures[:,0],1)
          if (type(w_r) == int) != True and np.any(ratios_1_r) == True:    
              line_error_logr=np.std(np.polyval(w_r,ratios_1[:,0])-temperatures[:,0]) 
              linear_r=[]
              lin2_r=[]    
              for number in np.linspace(min(ratios_1[:,0]),max(ratios_1[:,0]),100):
                linear_r.append(float(w_r[0])*number+float(w_r[1]))
                lin2_r.append(number)
              poli_logr=robust.polyfit(ratios_1[:,0],temperatures[:,0],3)
              ypred_log=(np.polyval(poli_logr,ratios_1[:,0])-temperatures[:,0])    
              logr_res=np.std(np.polyval(poli_logr,ratios_1[:,0])-temperatures[:,0])   
              if np.any(ratios_1)==True and np.any(ratios_1_r)==True:
                output.append((option,int(line_error_logr),len(temperatures[:,0]),data[0][0],data[0][1],get_element_read(line_list,float(data[0][0])),get_element_read(line_list,float(data[0][1])),format(w_r[1], '.2f'),format(w_r[0], '.2f'),format(0, '.2f'),format(0, '.2f'),int(float(min(temperatures[:,0]))),int(float(max(temperatures[:,0]))),format(min(np.power(10,ratios_1_r[:,0])), '.2f'),format(max(np.power(10,ratios_1_r[:,0])), '.2f'),format(np.mean(np.power(10,ratios_1_r[:,0])), '.2f'),5))
                output.append((option,int(logr_res),len(temperatures[:,0]),data[0][0],data[0][1],get_element_read(line_list,float(data[0][0])),get_element_read(line_list,float(data[0][1])),format(poli_logr[3], '.2f'),format(poli_logr[2], '.2f'),format(poli_logr[1], '.2f'),format(poli_logr[0], '.2f'),int(float(min(temperatures[:,0]))),int(float(max(temperatures[:,0]))),format(min(np.power(10,ratios_1_r[:,0])), '.2f'),format(max(np.power(10,ratios_1_r[:,0])), '.2f'),format(np.mean(np.power(10,ratios_1_r[:,0])), '.2f'),6))        
                plt.subplot(3,1,3)
                plt.errorbar(ratios_1[:,0],temperatures[:,0],yerr=temperatures[:,1],fmt='bo')  
                plt.plot(ratios_1[:,0],temperatures[:,0],'bo',lin2_r[:],linear_r[:],'r-',poliy_logr,poli2_logr(poliy_logr),'g-')
                plt.xlabel('log(R)',fontsize=20)  
                plt.ylabel('Teff (K)',fontsize=20)    
                plt.legend(['stars: '+str(len(temperatures[:,0])),'P_sigma: '+str("%.0f" % logr_res), 'L_sigma: '+str("%.0f" % line_error_logr)],prop={'size':12})
                plt.ylim((4300,7200))
                plt.xlim((min(ratios_1[:,0])-0.2*abs(min(ratios_1[:,0])),max(ratios_1[:,0])+0.2*abs(max(ratios_1[:,0]))))
  else:
      plt.plot(0,0)
  plt.subplots_adjust(wspace = 0.4)
  plt.savefig('./figures/'+str(data[0][0])+'_'+str(data[0][1]),format='png')      
  plt.close()    
  output=np.array([output[i] for i in range(0,len(output)) if output[i][1]<150 and output[i][2]>=math.floor((2./3)*len(data))])
  if output!=[]:
    a =  np.array(output[:][:,1],dtype=float)
    output=np.array([line1 for line1 in output if float(line1[1])== np.min(a)])
  else:
    output=np.array([[option,-1,-1,data[0][0],data[0][1],'A','B',-1,-1,-1,-1,-1,-1,-1,-1,-1,-1]])    
  return np.array(output)

       
def ultimate_plotter3(data,option,line_list,fe_data):
  i=0
  counter=0
  j=0
  bla=[]  
  
  if option==1:
      output_for_file=open('new_teff_cal','w')
      output_for_file.close()
      output_for_file=open('new_teff_cal','a')
      output_for_file.writelines('nf	ni	stddev	nstars	line1	line2	ele1	ele2	c0	c1	c2	c3  teffa teffb rata ratb meanr nfit\n')
      counter=0
      while i< len(data):          
        bla=ultimate_plotter2(data[i],i,line_list)
        if bla[0][2]!='-1':
          bla=([str(counter),str(bla[0][0]),str(bla[0][1]),str(bla[0][2]),str(bla[0][3]),str(bla[0][4]),str(bla[0][5]),str(bla[0][6]),str(bla[0][7]),str(bla[0][8]),str(bla[0][9]),str(bla[0][10]),str(bla[0][11]),str(bla[0][12]),str(bla[0][13]),str(bla[0][14]),str(bla[0][15]),str(bla[0][16])])
          bla=str(bla)
          output_for_file.writelines('%s\n' % bla.strip(' \'').strip('[').strip(']').replace(',','\t').replace('\'',''))
          counter+=1
          print len(data)-i,
          i+=1
        else:
          print len(data)-i,
          i+=1
      output_for_file.close()
   
  if option==2:
      output_for_fe=open('new_fe_cal','w')
      output_for_fe.close()      
      output_for_fe=open('new_fe_cal','a')
      output_for_fe.writelines('nl	stddev	nstars	line	ele	c0	c1	c2	c3	c4	teffa	teffb	feha	fehb	ewa	ewb	slope\n')
      while j<len(fe_data):
        bla=fit_fe_poli(fe_data[j],1,line_list)
        if str(bla[2])!='-1':
            bla=([str(counter+1),str(bla[0]),str(bla[1]),str(bla[2]),str(bla[3]),str('%e' % float(bla[4])),str('%e' % float(bla[5])),str('%e' % float(bla[6])),str('%e' % float(bla[7])),str('%e' % float(bla[8])),str('%e' % float(bla[9])),str(int(float(bla[10]))),str(int(float(bla[11]))),str(bla[12]),str(bla[13]),str("{0:.1f}".format(float(bla[14]))),str("{0:.1f}".format(float(bla[15]))),str(bla[16])])   
            bla=str(bla)
            output_for_fe.writelines('%s\n' % bla.strip(' \'').strip('[').strip(']').replace(',','\t').replace('\'',''))
            print len(fe_data)-j
            j+=1
            counter+=1
        else:
            print len(fe_data)-j
            j+=1
      output_for_fe.close()
      print 'finished producing fe_cal file with '+str(counter)+' lines'        
     
  if option==3:
      output_for_file=open('new_teff_cal','w')
      output_for_file.close()
      output_for_fe=open('new_fe_cal','w')
      output_for_fe.close()
      output_for_file=open('new_teff_cal','a')
      output_for_file.writelines('nf	ni	stddev	nstars	line1	line2	ele1	ele2	c0	c1	c2	c3  teffa teffb rata ratb meanr nfit\n')
      output_for_fe=open('new_fe_cal','a')
      output_for_fe.writelines('nl	stddev	nstars	line	ele	c0	c1	c2	c3	c4	teffa	teffb	feha	fehb	ewa	ewb	slope\n')
      while j<len(fe_data):
        bla=fit_fe_poli(fe_data[j],1,line_list)
        if str(bla[2])!='-1':
            bla=([str(counter+1),str(bla[0]),str(bla[1]),str(bla[2]),str(bla[3]),str('%e' % float(bla[4])),str('%e' % float(bla[5])),str('%e' % float(bla[6])),str('%e' % float(bla[7])),str('%e' % float(bla[8])),str('%e' % float(bla[9])),str(int(float(bla[10]))),str(int(float(bla[11]))),str(bla[12]),str(bla[13]),str("{0:.1f}".format(float(bla[14]))),str("{0:.1f}".format(float(bla[15]))),str(bla[16])])   
            bla=str(bla)
            output_for_fe.writelines('%s\n' % bla.strip(' \'').strip('[').strip(']').replace(',','\t').replace('\'',''))
            print len(fe_data)+len(data)-j
            j+=1
            counter+=1
        else:
            print len(fe_data)+len(data)-j
            j+=1
      output_for_fe.close()
      print 'finished producing fe_cal file with '+str(counter)+' lines'
      bla=[]    
      counter=0
      while i< len(data):            
        bla=ultimate_plotter2(data[i],i,line_list)
        if bla[0][2]!='-1':
          bla=([str(counter),str(bla[0][0]),str(bla[0][1]),str(bla[0][2]),str(bla[0][3]),str(bla[0][4]),str(bla[0][5]),str(bla[0][6]),str(bla[0][7]),str(bla[0][8]),str(bla[0][9]),str(bla[0][10]),str(bla[0][11]),str(bla[0][12]),str(bla[0][13]),str(bla[0][14]),str(bla[0][15]),str(bla[0][16])])
          bla=str(bla)
          output_for_file.writelines('%s\n' % bla.strip(' \'').strip('[').strip(']').replace(',','\t').replace('\'',''))
          counter+=1
          print len(fe_data)+len(data)-i-j
          i+=1
        else:
          print len(fe_data)+len(data)-i-j  
          i+=1
      output_for_file.close()
  



def fit_fe_poli(data,option,line_list):
    plt.figure(figsize=(10,15)) 
    EW=np.array([np.array((float(data[i][1]),1)) for i in range(0,len(data)) if float(data[i][1]) != -1 and float(data[i][1]) < 75 and float(data[i][1]) > 20])
    temperatures=np.array([np.array((float(data[i][2]),float(data[i][3]))) for i in range(0,len(data)) if float(data[i][1]) != -1 and float(data[i][1]) < 75 and float(data[i][1]) > 20])
    metalic=np.array([np.array((float(data[i][8]),float(data[i][9]))) for i in range(0,len(data)) if float(data[i][1]) != -1 and float(data[i][1]) < 75 and float(data[i][1]) > 20])    
    if np.any(EW)==True:
        quart1=stats.scoreatpercentile(EW[:,0],25)
        quart3=stats.scoreatpercentile(EW[:,0],75)
        iqr=quart3-quart1
        lowqr=quart1-1.5*iqr
        highqr=quart3+1.5*iqr
        EW_temp=EW      
        metalic=np.array([metalic[i,:] for i in range(0,len(metalic)) if EW[i,0] <highqr and EW[i,0] > lowqr])      
        temperatures=np.array([temperatures[i,:] for i in range(0,len(temperatures)) if EW_temp[i,0] <highqr and EW_temp[i,0] > lowqr])
        EW=np.array([EW[i,:] for i in range(0,len(EW)) if EW_temp[i,0] <highqr and EW_temp[i,0] > lowqr])
        if np.any(metalic)==True and np.any(temperatures)==True and np.any(EW)==True:
            x=metalic[:,0]
            y=temperatures[:,0]
            z=EW[:,0] 
            degree=2
            A = []    
            for i in range(len(x)):
                A.append([])
                for xd in range(degree+1):
                    for yd in range(degree+1-xd):
                        A[i].append((x[i]**xd)*(y[i]**yd))     
            if np.any(A)==True:        
                c=[]
                c=linalg.lstsq(A,z)[0]                              
                ypred=x-(-(c[3]+c[4]*y)+np.sqrt(((c[3]+c[4]*y)**2-4*(c[5])*(c[0]+c[1]*y+c[2]*y**2-z))))/(2*c[5])
                y_res=np.std(ypred)
                z_old=z
                x=np.array([x[i] for i in range(0,len(x)) if abs((ypred[i])) <= 2*y_res])   
                z_cut=np.array([z[i] for i in range(0,len(z)) if abs((ypred[i])) > 2*y_res])
                ypred_cut=np.array([ypred[i] for i in range(0,len(y)) if abs((ypred[i])) > 2*y_res])
                y=np.array([y[i] for i in range(0,len(y)) if abs((ypred[i])) <= 2*y_res])
                z=np.array([z[i] for i in range(0,len(z)) if abs((ypred[i])) <= 2*y_res]) 
                plt.subplot(3,1,1)    
                plt.plot(z_old,ypred,'bx',[20,80],[-2*y_res,-2*y_res],'r--',[20,80],[2*y_res,2*y_res],'r--',[20,80],[0,0],'k-',z_cut,ypred_cut,'go')
                plt.title('Spectral line: '+str(data[0][0]),fontsize=20)   
                plt.ylabel('[Fe/H]_spec-[Fe/H]_cal (dex)',fontsize=18) 
                plt.legend(['Stars: '+str(len(z_old)), 'sigma: '+str("%.2f" % y_res)],prop={'size':20})
                plt.ylim((-(max(ypred)+0.2),max(ypred)+0.2))
                plt.xlim((20,80))
            A = []   
            for i in range(len(x)):
                A.append([])
                for xd in range(degree+1):
                    for yd in range(degree+1-xd):
                        A[i].append((x[i]**xd)*(y[i]**yd))      
            if np.any(A)==True:
                c=[]
                c=linalg.lstsq(A,z)[0]        
                ypred=x-(-(c[3]+c[4]*y)+np.sqrt(((c[3]+c[4]*y)**2-4*(c[5])*(c[0]+c[1]*y+c[2]*y**2-z))))/(2*c[5])
                y_res=np.std(ypred)
                z_old=z
                x=np.array([x[i] for i in range(0,len(x)) if abs((ypred[i])) <= 2*y_res])      

                z_cut=np.array([z[i] for i in range(0,len(z)) if abs((ypred[i])) > 2*y_res])
                ypred_cut=np.array([ypred[i] for i in range(0,len(y)) if abs((ypred[i])) > 2*y_res])
                y=np.array([y[i] for i in range(0,len(y)) if abs((ypred[i])) <= 2*y_res])
                z=np.array([z[i] for i in range(0,len(z)) if abs((ypred[i])) <= 2*y_res])
                plt.subplot(3,1,2)   
                plt.plot(z_old,ypred,'bx',[20,80],[-2*y_res,-2*y_res],'r--',[20,80],[2*y_res,2*y_res],'r--',[20,80],[0,0],'k-',z_cut,ypred_cut,'go')
                plt.ylabel('[Fe/H]_spec-[Fe/H]_cal (dex)',fontsize=18)     
                plt.legend(['Stars: '+str(len(z_old)), 'sigma: '+str("%.2f" % y_res)],prop={'size':20})
                plt.ylim((-(max(ypred)+0.2),max(ypred)+0.2))
                plt.xlim((20,80))                
            A = []    
            for i in range(len(x)):
                A.append([])
                for xd in range(degree+1):
                    for yd in range(degree+1-xd):
                        A[i].append((x[i]**xd)*(y[i]**yd))     
            if np.any(A)==True:            
                c=[]
                c=linalg.lstsq(A,z)[0]    
                x_pred=(-(c[3]+c[4]*y)+np.sqrt(((c[3]+c[4]*y)**2-4*(c[5])*(c[0]+c[1]*y+c[2]*y**2-z))))/(2*c[5])    
                y_res=np.std(x-x_pred)
                slope, intercept_logr, r_value_logr, p_value_logr, std_err_logr = stats.linregress(x,x_pred)
                plt.subplot(3,1,3)
                plt.plot(z,x-x_pred,'bx',[20,80],[-2*y_res,-2*y_res],'r--',[20,80],[2*y_res,2*y_res],'r--',[20,80],[0,0],'k-')
                plt.ylabel('[Fe/H]_spec-[Fe/H]_cal (dex)',fontsize=18) 
                plt.xlabel('EW (mA)',fontsize=20)
                plt.legend(['Stars: '+str(len(x)), 'sigma: '+str("%.2f" % y_res)],prop={'size':20})
                plt.ylim((-(max(ypred)+0.2),max(ypred)+0.2))
                plt.xlim((20,80))                
                if y_res < 0.06 and slope<1.03 and slope>0.97 and len(x)>=math.floor(0*len(data)):
                    output=np.array((y_res,len(x), data[0][0], str('FeI'),str(c[0]), str(c[3]), str(c[1]), str(c[5]), str(c[2]), str(c[4]), np.min(y), np.max(y), np.min(x), np.max(x),np.min(z),np.max(z),slope))    
                    plt.savefig('./figures/fe_cal_'+str(data[0][0]),format='png')      
                else:
                    output=np.array((-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1))
            else:
               output=np.array((-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1)) 
        else:
           output=np.array((-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1))  
    else:
        output=np.array((-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1))
    plt.close()
    return output

def option_choose(list_of_ares_files,params_file,line_list,option):
    list_of_params = param_reader(params_file)
    if not os.path.exists('./figures'):
            os.makedirs('./figures') 
    if option==1:
        print "You have choosen to calibrate only the temperatures!"
        all_rat = allow_ratios(line_list,line_list)
        print "all_rat.................................... done"
        list_of_lists_ratios = [ratiocalc(ares_output_reader(str(line).strip('\n')),all_rat) for line in open(list_of_ares_files,'r')]
        print "list_of_lists_ratios....................... done"
        ordered_lists = [[list_of_lists_ratios[j][i,:]  for j in range(0,len(list_of_lists_ratios))] for i in range(0,len(all_rat))]
        print "ordered_lists.............................. done"
        list_of_plotting_data = [[np.hstack([ordered_lists[i][j],list_of_params[j]]) for j in range(0,len(list_of_params))] for i in range(0,len(all_rat))]
        print "list_of_plotting_data...................... done"
        list_of_fe_data = []
        ultimate_plotter3(list_of_plotting_data,1,line_list,list_of_fe_data)
        print "Temperature calibration.................... done"
    if option==2:
        print "You have choosen to calibrate only the metallicity!"
        allowed_fe=allow_fe(line_list)
        print "allowed_fe................................. done"
        list_of_lists_fe = [fe_aux(ares_output_reader(str(line).strip('\n')),allowed_fe) for line in open(list_of_ares_files,'r')]
        print "list_of_lists_fe........................... done"
        ordered_lists_fe = [[list_of_lists_fe[j][i,:]  for j in range(0,len(list_of_lists_fe))] for i in range(0,len(allowed_fe))]
        print "ordered_lists_fe........................... done"
        list_of_fe_data = [[np.hstack([ordered_lists_fe[i][j],list_of_params[j]]) for j in range(0,len(list_of_params))] for i in range(0,len(allowed_fe))]
        print "list_of_fe_data............................ done" 
        list_of_plotting_data=[]
        ultimate_plotter3(list_of_plotting_data,2,line_list,list_of_fe_data)
        print "Metallicity calibration.................... done"
    if option==3:
        print "You have choosen to calibrate the temperature and metallicity!"
        all_rat = allow_ratios(line_list,line_list)
        print "all_rat.................................... done"
        list_of_lists_ratios = [ratiocalc(ares_output_reader(str(line).strip('\n')),all_rat) for line in open(list_of_ares_files,'r')]
        print "list_of_lists_ratios....................... done"
        ordered_lists = [[list_of_lists_ratios[j][i,:]  for j in range(0,len(list_of_lists_ratios))] for i in range(0,len(all_rat))]
        print "ordered_lists.............................. done"
        list_of_plotting_data = [[np.hstack([ordered_lists[i][j],list_of_params[j]]) for j in range(0,len(list_of_params))] for i in range(0,len(all_rat))]
        print "list_of_plotting_data...................... done"
        allowed_fe=allow_fe(line_list)
        print "allowed_fe................................. done"
        list_of_lists_fe = [fe_aux(ares_output_reader(str(line).strip('\n')),allowed_fe) for line in open(list_of_ares_files,'r')]
        print "list_of_lists_fe........................... done"
        ordered_lists_fe = [[list_of_lists_fe[j][i,:]  for j in range(0,len(list_of_lists_fe))] for i in range(0,len(allowed_fe))]
        print "ordered_lists_fe........................... done"
        list_of_fe_data = [[np.hstack([ordered_lists_fe[i][j],list_of_params[j]]) for j in range(0,len(list_of_params))] for i in range(0,len(allowed_fe))]
        print "list_of_fe_data............................ done" 
        ultimate_plotter3(list_of_plotting_data,3,line_list,list_of_fe_data)
        print "Temperature and metallicity calibration.... done"

                            #############
                            ###  MAIN  ##
                            #############

print " "
print "===================="
print "||INPUT PARAMETERS||"
print "===================="
print "LIST OF ARES FILES: "+str(list_of_ares_files_tt)
print "PARAMETERS FILE: "+str(params_file_tt)
print "LINE LIST: "+str(line_list_tt)
print "OPTION: "+str(option_tt)
print " "
print "Initializing......"
option_choose(str(list_of_ares_files_tt),str(params_file_tt),str(line_list_tt),int(option_tt))